#!/bin/env python3
"""
Process a sample B into microsamples to reproduce A

Copyright (c) 2024, Anzo

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.
"""

import numpy as np
import scipy.io.wavfile as wv
import os
import sys

# Args 
import argparse
parser = argparse.ArgumentParser(description="Process a source sample \
        in microsamples, rebuild to match \
        a target sample")

parser.add_argument('source')
parser.add_argument('target')
parser.add_argument('--sample-amount','-n',default=900, type=int
                    , help="amount of sample used in each microsamples")
parser.add_argument('--output','-o',default="out.wav"
                    , help="output name of the resulting file")

def MeanSquare(A,B):
    if len(A) != len(B):
        raise ValueError("List aren't the same dimension")
    res = (np.array((A-B),dtype=np.int32)**2).mean()
    if res < 0:
        print("overflow error")
    return res

def FindBestSlice(ref, source):
    besti = 0
    n = len(ref)
    bestval = MeanSquare(ref, source[:n])
    for i in range(1,len(source)-n+1):
        mean = MeanSquare(ref,source[i:i+n])
        if (mean < bestval):
            besti = i
            bestval = mean
            #print("new value:"+str(bestval)+" at "+str(besti))
    return besti

def ConstructNewSample(ref, source, n):
    lenRef = len(ref) 
    res = np.zeros(lenRef,dtype=ref.dtype) 
    i = 0
    while i < (lenRef-1-n-n):
        bestidx = FindBestSlice(ref[i:i+n], source)
        print(bestidx)
        res[i:i+n] = source[bestidx:bestidx+n]
        i = i+n
        print("\033[2AProcessed "+str(i)+" sample points over "
              +str(lenRef)+" sample points...")
    return res 

if __name__ == '__main__':
    args = parser.parse_args()
    print(args)
    nsample = args.sample_amount
    outname = args.output
    samplertT, audioTarget = wv.read(args.target)
    samplertS, audioSource = wv.read(args.source)
    
    #print(MeanSquare([-555,-0,-2,-6],[1, 2, 3, 4]))
    print(audioTarget.dtype, samplertS)
    print(audioSource.dtype, samplertS)
    print("")
    out = ConstructNewSample(audioTarget[:,0], audioSource[:,0], nsample)
    
    print("Processing done, writing file...")
    wv.write(outname,samplertT,out)
